package io.zipcoder.tc_spring_poll_application.controller;

import io.zipcoder.tc_spring_poll_application.domain.Poll;
import io.zipcoder.tc_spring_poll_application.domain.Vote;
import io.zipcoder.tc_spring_poll_application.dtos.OptionCount;
import io.zipcoder.tc_spring_poll_application.dtos.VoteResult;
import io.zipcoder.tc_spring_poll_application.exception.ResourceNotFoundException;
import io.zipcoder.tc_spring_poll_application.repositories.PollRepository;
import io.zipcoder.tc_spring_poll_application.repositories.VoteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

@RestController
public class ComputeResultController {

    private VoteRepository voteRepository;
    private PollRepository pollRepository;

    @Autowired
    public ComputeResultController(VoteRepository voteRepository, PollRepository pollRepository) {
        this.voteRepository = voteRepository;
        this.pollRepository = pollRepository;
    }

    @RequestMapping(value = "/computeresult", method = RequestMethod.GET)
    public ResponseEntity<?> computeResult(@RequestParam Long pollId) {
        verifyPoll(pollId);
        VoteResult voteResult = new VoteResult();
        voteResult.setResults(new ArrayList<>());
        Collection<OptionCount> results = voteResult.getResults();

        Iterable<Vote> allVotes = voteRepository.findVotesByPoll(pollId);
        Iterator<Vote> voteIterator = allVotes.iterator();

        while (voteIterator.hasNext()){
            Vote vote = voteIterator.next();
            if (results.size() == 0) {
                results.add(newOptionCount(vote));
            }

            else {
                boolean added = false;
                for (OptionCount oc : results) {
                    if (oc.getOptionId() == vote.getOption().getId()) {
                        oc.setCount(oc.getCount() + 1);
                        added = true;
                        break;
                    }
                }
                if (!added) results.add(newOptionCount(vote));
            }
        }

        return new ResponseEntity<>(results, HttpStatus.OK);
    }

    private OptionCount newOptionCount(Vote vote) {
        OptionCount oc = new OptionCount();
        oc.setCount(1);
        oc.setoptionId(vote.getOption().getId());
        return oc;
    }

    public void verifyPoll(Long pollId){
        Poll p = pollRepository.findOne(pollId);
        if (p == null) throw new ResourceNotFoundException();
    }

}
