package io.zipcoder.tc_spring_poll_application.controller;

import io.zipcoder.tc_spring_poll_application.domain.Poll;
import io.zipcoder.tc_spring_poll_application.domain.Vote;
import io.zipcoder.tc_spring_poll_application.exception.ResourceNotFoundException;
import io.zipcoder.tc_spring_poll_application.repositories.PollRepository;
import io.zipcoder.tc_spring_poll_application.repositories.VoteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import java.net.URI;

@RestController
public class VoteController {
    VoteRepository voteRepository;
    PollRepository pollRepository;

    @Autowired
    public VoteController(VoteRepository voteRepository, PollRepository pollRepository){
        this.voteRepository = voteRepository;
        this.pollRepository = pollRepository;
    }

    @RequestMapping(value = "/polls/votes", method = RequestMethod.GET)
    public Iterable<Vote> getAllVotes(){
        return voteRepository.findAll();
    }

    @RequestMapping(value = "/polls/votes", params = { "page", "size" }, method = RequestMethod.GET)
    public Iterable<Vote> getPaginatedVotes(@RequestParam("page") int page, @RequestParam("size") int size) {
        return voteRepository.findAll(new PageRequest(page,size));
    }

    @RequestMapping(value = "/polls/{pollId}/votes", method = RequestMethod.GET)
    public Iterable<Vote> getVote(@PathVariable Long pollId) {
        verifyPoll(pollId);
        return voteRepository.findVotesByPoll(pollId);
    }

    @RequestMapping(value = "/polls/{pollId}/votes", method= RequestMethod.POST)
    public ResponseEntity<?> createVote(@PathVariable Long pollId, @RequestBody Vote vote){
        verifyPoll(pollId);
        vote = voteRepository.save(vote);

        URI newVoteUri = ServletUriComponentsBuilder
                .fromCurrentRequest()
                .path("/{id}")
                .buildAndExpand(vote.getId())
                .toUri();

        HttpHeaders responseHeaders = new HttpHeaders();
        responseHeaders.setLocation(newVoteUri);
        return new ResponseEntity<>(null, responseHeaders, HttpStatus.CREATED);
    }

    public void verifyPoll(Long pollId){
        Poll p = pollRepository.findOne(pollId);
        if (p == null) throw new ResourceNotFoundException();
    }

}
